%top{
/*
 *  Scanner for C declarations.
 *  Copyright © 2011 Nick Bowler
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

 #include "parse.h"
}

%option noyywrap bison-locations

%{
#define lex_error(msg) do { \
	yyerror(yylloc, NULL, (msg)); \
	return T_LEX_ERROR; \
} while(0)
%}

IDENT [_[:alpha:]][_[:alnum:]]*

%%

";" return T_SEMICOLON;
"*" return T_ASTERISK;
"(" return T_LPAREN;
")" return T_RPAREN;
"[" return T_LBRACKET;
"]" return T_RBRACKET;
"," return T_COMMA;

"typedef"  return T_TYPEDEF;
"extern"   return T_EXTERN;
"static"   return T_STATIC;
"auto"     return T_AUTO;
"register" return T_REGISTER;

"restrict" return T_RESTRICT;
"volatile" return T_VOLATILE;
"const"    return T_CONST;

"inline"   return T_INLINE;

"void"     return T_VOID;
"char"     return T_CHAR;
"short"    return T_SHORT;
"int"      return T_INT;
"long"     return T_LONG;
"float"    return T_FLOAT;
"double"   return T_DOUBLE;
"signed"   return T_SIGNED;
"unsigned" return T_UNSIGNED;
"_Bool"    return T_BOOL;
"_Complex" return T_COMPLEX;

"struct"   return T_STRUCT;
"union"    return T_UNION;
"enum"     return T_ENUM;

{IDENT} {
	yylval->strval = malloc(yyleng+1);
	if (!yylval->strval)
		lex_error("failed to allocate memory");

	strcpy(yylval->strval, yytext);
	return T_IDENT;
}

[[:space:]]+
. {
	char buf[] = "syntax error, unexpected #";
	*strchr(buf, '#') = *yytext;
	lex_error(buf);
}
